var Endpoint;
(function (Endpoint) {
    Endpoint[Endpoint["FileList"] = 0] = "FileList";
    Endpoint[Endpoint["FileEdit"] = 1] = "FileEdit";
    Endpoint[Endpoint["DocRead"] = 2] = "DocRead";
    Endpoint[Endpoint["DocCheckForUpdates"] = 3] = "DocCheckForUpdates";
    Endpoint[Endpoint["DocEdit"] = 4] = "DocEdit";
    Endpoint[Endpoint["InboxAdd"] = 5] = "InboxAdd";
    Endpoint[Endpoint["Upload"] = 6] = "Upload";
    Endpoint[Endpoint["PrefGet"] = 7] = "PrefGet";
    Endpoint[Endpoint["PrefSet"] = 8] = "PrefSet";
})(Endpoint || (Endpoint = {}));
var Endpoint$1 = Endpoint;

const API_URL = new URL('https://dynalist.io/api/v1/');
const FILE_API_URL = new URL('file/', API_URL);
const DOC_API_URL = new URL('doc/', API_URL);
const INBOX_API_URL = new URL('inbox/', API_URL);
const PREF_API_URL = new URL('pref/', API_URL);
var endpointToUrl = {
    [Endpoint$1.FileList]: new URL('list', FILE_API_URL),
    [Endpoint$1.FileEdit]: new URL('edit', FILE_API_URL),
    [Endpoint$1.DocRead]: new URL('read', DOC_API_URL),
    [Endpoint$1.DocCheckForUpdates]: new URL('check_for_updates', DOC_API_URL),
    [Endpoint$1.DocEdit]: new URL('edit', DOC_API_URL),
    [Endpoint$1.InboxAdd]: new URL('add', INBOX_API_URL),
    [Endpoint$1.Upload]: new URL('upload', API_URL),
    [Endpoint$1.PrefGet]: new URL('get', PREF_API_URL),
    [Endpoint$1.PrefSet]: new URL('set', PREF_API_URL),
};

class Client {
    /** @param token Your API secret token. */
    constructor(token) {
        this.token = token;
    }
    /** Lists all your documents and folders. */
    listDocumentsAndFolders() {
        return this.requestEndpoint(Endpoint$1.FileList);
    }
    /** Changes (moves, edits or creates) specific documents and folders. */
    changeDocumentsAndFolders(changes) {
        return this.requestEndpoint(Endpoint$1.FileEdit, { changes });
    }
    moveDocumentOrFolder(moveParams) {
        return this.changeDocumentOrFolder('move', moveParams);
    }
    editDocumentOrFolder(editParams) {
        return this.changeDocumentOrFolder('edit', editParams);
    }
    createDocumentOrFolder(createParams) {
        return this.changeDocumentOrFolder('create', createParams);
    }
    /** Fetches the content of a specific document. */
    readDocument(documentId) {
        return this.requestEndpoint(Endpoint$1.DocRead, { file_id: documentId });
    }
    /** Fetches the latest version numbers of specific documents. */
    fetchDocumentsVersionNumbers(documentIds) {
        return this.requestEndpoint(Endpoint$1.DocCheckForUpdates, { file_ids: documentIds });
    }
    /** Fetches the latest version number of a specific document. */
    async fetchDocumentVersionNumber(documentId) {
        const response = await this.fetchDocumentsVersionNumbers([documentId]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            version: response.versions[documentId],
        };
    }
    /** Changes the content (inserts, edits, moves or deletes nodes) of a specific document. */
    changeDocumentContent(documentId, changes) {
        return this.requestEndpoint(Endpoint$1.DocEdit, { file_id: documentId, changes });
    }
    insertNode(documentId, insertParams) {
        return this.makeOneNodeChange(documentId, 'insert', insertParams);
    }
    editNode(documentId, editParams) {
        return this.makeOneNodeChange(documentId, 'edit', editParams);
    }
    moveNode(documentId, moveParams) {
        return this.makeOneNodeChange(documentId, 'move', moveParams);
    }
    deleteNode(documentId, deleteParams) {
        return this.makeOneNodeChange(documentId, 'delete', deleteParams);
    }
    /** Adds a node to your inbox. */
    addToInbox(params) {
        return this.requestEndpoint(Endpoint$1.InboxAdd, params);
    }
    /** Uploads a file. (Pro only) */
    uploadFile(params) {
        return this.requestEndpoint(Endpoint$1.Upload, params);
    }
    /** Gets a user preference value. */
    getPreference(preferenceKey) {
        return this.requestEndpoint(Endpoint$1.PrefGet, { key: preferenceKey });
    }
    /** Sets a user preference value. */
    setPreference(params) {
        return this.requestEndpoint(Endpoint$1.PrefSet, params);
    }
    async changeDocumentOrFolder(action, change) {
        const response = await this.changeDocumentsAndFolders([{ action, ...change }]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            ...(action !== 'create' ? { succeeded: response.results[0] } : { created_file_id: response.created[0] }),
        };
    }
    async makeOneNodeChange(documentId, action, change) {
        const response = await this.changeDocumentContent(documentId, [{ action, ...change }]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            ...(action === 'insert' ? { new_node_id: response.new_node_ids[0] } : {}),
        };
    }
    async requestEndpoint(endpoint, parameters) {
        const response = await fetch(endpointToUrl[endpoint].href, {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({
                token: this.token,
                ...parameters,
            }),
        });
        return response.json();
    }
}

/**
 * Converts an array of nodes which have an id and their children's ids, to a tree.
 * The root is the node with the id `rootId`. If it doesn't exist return null.
 */
function buildTree(arrayNodes, rootId) {
    if (!arrayNodes.some(({ id }) => id === rootId))
        return null;
    const idToArrayNode = {};
    arrayNodes.forEach((node) => { idToArrayNode[node.id] = node; });
    return internalBuildTree(idToArrayNode, rootId);
}
function internalBuildTree(idToArrayNode, rootId) {
    const rootNode = idToArrayNode[rootId];
    return !rootNode.children ? rootNode : {
        ...rootNode,
        ...{ children: rootNode.children.map((id) => internalBuildTree(idToArrayNode, id)) },
    };
}

export { Client, buildTree };
