import 'cross-fetch/polyfill';
import Endpoint from './endpoint.js';
import endpointToUrl from './endpoint-to-url.js';

class Client {
    /** @param token Your API secret token. */
    constructor(token) {
        this.token = token;
    }
    /** Lists all your documents and folders. */
    listDocumentsAndFolders() {
        return this.requestEndpoint(Endpoint.FileList);
    }
    /** Changes (moves, edits or creates) specific documents and folders. */
    changeDocumentsAndFolders(changes) {
        return this.requestEndpoint(Endpoint.FileEdit, { changes });
    }
    moveDocumentOrFolder(moveParams) {
        return this.changeDocumentOrFolder('move', moveParams);
    }
    editDocumentOrFolder(editParams) {
        return this.changeDocumentOrFolder('edit', editParams);
    }
    createDocumentOrFolder(createParams) {
        return this.changeDocumentOrFolder('create', createParams);
    }
    /** Fetches the content of a specific document. */
    readDocument(documentId) {
        return this.requestEndpoint(Endpoint.DocRead, { file_id: documentId });
    }
    /** Fetches the latest version numbers of specific documents. */
    fetchDocumentsVersionNumbers(documentIds) {
        return this.requestEndpoint(Endpoint.DocCheckForUpdates, { file_ids: documentIds });
    }
    /** Fetches the latest version number of a specific document. */
    async fetchDocumentVersionNumber(documentId) {
        const response = await this.fetchDocumentsVersionNumbers([documentId]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            version: response.versions[documentId],
        };
    }
    /** Changes the content (inserts, edits, moves or deletes nodes) of a specific document. */
    changeDocumentContent(documentId, changes) {
        return this.requestEndpoint(Endpoint.DocEdit, { file_id: documentId, changes });
    }
    insertNode(documentId, insertParams) {
        return this.makeOneNodeChange(documentId, 'insert', insertParams);
    }
    editNode(documentId, editParams) {
        return this.makeOneNodeChange(documentId, 'edit', editParams);
    }
    moveNode(documentId, moveParams) {
        return this.makeOneNodeChange(documentId, 'move', moveParams);
    }
    deleteNode(documentId, deleteParams) {
        return this.makeOneNodeChange(documentId, 'delete', deleteParams);
    }
    /** Adds a node to your inbox. */
    addToInbox(params) {
        return this.requestEndpoint(Endpoint.InboxAdd, params);
    }
    /** Uploads a file. (Pro only) */
    uploadFile(params) {
        return this.requestEndpoint(Endpoint.Upload, params);
    }
    /** Gets a user preference value. */
    getPreference(preferenceKey) {
        return this.requestEndpoint(Endpoint.PrefGet, { key: preferenceKey });
    }
    /** Sets a user preference value. */
    setPreference(params) {
        return this.requestEndpoint(Endpoint.PrefSet, params);
    }
    async changeDocumentOrFolder(action, change) {
        const response = await this.changeDocumentsAndFolders([{ action, ...change }]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            ...(action !== 'create' ? { succeeded: response.results[0] } : { created_file_id: response.created[0] }),
        };
    }
    async makeOneNodeChange(documentId, action, change) {
        const response = await this.changeDocumentContent(documentId, [{ action, ...change }]);
        return response._code !== 'Ok' ? response : {
            _code: 'Ok',
            ...(action === 'insert' ? { new_node_id: response.new_node_ids[0] } : {}),
        };
    }
    async requestEndpoint(endpoint, parameters) {
        const response = await fetch(endpointToUrl[endpoint].href, {
            method: 'POST',
            headers: { 'Content-Type': 'application/json' },
            body: JSON.stringify({
                token: this.token,
                ...parameters,
            }),
        });
        return response.json();
    }
}

export default Client;
